import argparse
import asyncio
import json
import logging
import traceback

import aiohttp

from paasta_tools import marathon_tools
from paasta_tools import mesos_tools
from paasta_tools import tron_tools
from paasta_tools import utils

log = logging.getLogger(__name__)

LOG_COMPONENT = "task_lifecycle"


class UnknownFrameworkError(Exception):
    pass


def interpret_task_updated(task_updated) -> str:
    message = task_updated["status"].get("message")
    healthy = task_updated["status"].get("healthy")
    reason = task_updated["status"].get("reason")
    state = task_updated["state"]

    if state == "TASK_STARTING":
        return state
    if state == "TASK_RUNNING":
        if reason == "REASON_TASK_HEALTH_CHECK_STATUS_UPDATED":
            return f"Health check update: healthy={healthy}"
        elif reason is None:
            return f"Task running, no healthchecks defined."
        else:
            return "Unknown: TASK_RUNNING but unrecognized status.reason"
    if state == "TASK_KILLED":
        if healthy is False:
            message = task_updated["status"]["message"]
            if message == "Container exited with status 143":
                return "Killed by Mesos due to healthcheck failures"
            else:
                return f"Unknown: TASK_KILLED with healthy={healthy!r} but unrecognized status.message"
        elif healthy is None:
            return "Killed by Paasta (bounce? autoscaling?)"
        else:
            return f"Unknown: TASK_KILLED but unrecognized status.healthy={healthy!r}"
    elif state == "TASK_FAILED":
        if message == "Container exited with status 137":
            return f"Probable OOM: {message}"
        elif message is None:
            return f"Unknown: TASK_FAILED but status.message is None. This seems to happen when a task exits very quickly on startup. Mesos usually follows up with a corrected message shortly."
        elif message.startswith("Container exited with status "):
            return f"TASK_FAILED: {message}"
        else:
            return f"Unknown: TASK_FAILED but unrecognized status.message"
    elif state == "TASK_FINISHED":
        if message is None:
            return f"Unknown: TASK_FINISHED but status.message is None. This seems to happen when a task exits very quickly on startup. Mesos usually follows up with a corrected message shortly."
        return f"TASK_FINISHED: {message}"
    else:
        return f"Unknown: unrecognized state"


class MesosEventSubscriber:
    def __init__(self, cluster):
        self.framework_id_to_name = {}
        self.framework_id_task_id_to_git_sha = {}
        self.cluster = cluster

    async def subscribe(self):
        # This connection should live ~forever, so disable some timeouts.
        timeout = aiohttp.ClientTimeout(
            total=None,
            sock_read=None,
            connect=30,
            sock_connect=30,
        )
        async with aiohttp.ClientSession(timeout=timeout) as session:
            payload = '{"type":"SUBSCRIBE"}'
            master_host_port = mesos_tools.find_mesos_leader(cluster=self.cluster)
            async with session.post(
                f"http://{master_host_port}/api/v1",
                data=payload,
                # allow_redirects=True,
                headers={"Content-Type": "application/json"},
                timeout=timeout,
            ) as resp:
                while True:
                    _size = await resp.content.readline()
                    if not _size:
                        break
                    size = int(_size)
                    record = await resp.content.readexactly(size)
                    yield json.loads(record)

    def determine_service_instance(self, framework_name, status):
        executor_id = status.get("executor_id", {}).get("value")
        task_id = status.get("task_id", {}).get("value")

        if framework_name.startswith("marathon"):
            return marathon_tools.parse_service_instance_from_executor_id(
                executor_id or task_id
            )
        elif framework_name.startswith("tron"):
            return tron_tools.decompose_executor_id(executor_id)[:2]
        elif framework_name.startswith("paasta-remote "):
            # sorta gross, but it's the same format.
            return marathon_tools.parse_service_instance_from_executor_id(executor_id)
        else:
            raise UnknownFrameworkError(
                f"don't know how to parse task IDs for framework {framework_name}"
            )

    def skip_updates_from_framework(self, framework_name) -> bool:
        if framework_name.startswith("jupyterhub_"):
            return True
        if framework_name.startswith("jenkins"):
            return True
        if framework_name.startswith("paasta_spark_run"):
            return True
        return False

    def handler_task_updated(self, event):
        task_updated = event["task_updated"]
        try:
            del task_updated["status"]["data"]
        except KeyError:
            pass

        framework_name = self.framework_id_to_name[
            task_updated["framework_id"]["value"]
        ]
        if self.skip_updates_from_framework(framework_name):
            return

        service, instance = self.determine_service_instance(
            framework_name, task_updated["status"]
        )
        git_sha = self.framework_id_task_id_to_git_sha.get(
            (
                task_updated["framework_id"]["value"],
                task_updated["status"]["task_id"]["value"],
            )
        )

        self.log_task_updated(
            service=service,
            instance=instance,
            git_sha=git_sha,
            task_updated=task_updated,
        )

    def log_task_updated(self, service, instance, git_sha, task_updated):
        message = {
            "type": "mesos_task_updated",
            "is_terminal": task_updated["state"] in mesos_tools.TERMINAL_STATES,
            "interpretation": interpret_task_updated(task_updated),
            "git_sha": git_sha,
            "task_updated": task_updated,
        }
        utils._log(
            service=service,
            instance=instance,
            component=LOG_COMPONENT,
            cluster=self.cluster,
            line=json.dumps(message),
        )

    def handler_subscribed(self, event):
        state = event["subscribed"]["get_state"]
        for framework in state["get_frameworks"]["frameworks"]:
            framework_info = framework["framework_info"]
            self.register_framework(framework_info)
        for task in state["get_tasks"]["tasks"]:
            self.register_task(task)

    def register_framework(self, framework_info):
        self.framework_id_to_name[framework_info["id"]["value"]] = framework_info[
            "name"
        ]

    def register_task(self, task):
        framework_name = self.framework_id_to_name[task["framework_id"]["value"]]
        if self.skip_updates_from_framework(framework_name):
            return
        git_sha = self.get_git_sha_from_task_dict(task)
        self.framework_id_task_id_to_git_sha[
            task["framework_id"]["value"], task["task_id"]["value"]
        ] = git_sha

    def get_git_sha_from_task_dict(self, task):
        try:
            docker_image = task["container"]["docker"]["image"]
        except KeyError:
            try:
                docker_image = task["container"]["mesos"]["image"]["docker"]["name"]
            except KeyError:
                log.debug("Could not figure out what docker image task uses: {task}")
                return None

        return utils.get_git_sha_from_dockerurl(docker_image)

    def handler_framework_added(self, event):
        self.register_framework(event["framework_added"]["framework"]["framework_info"])

    def handler_task_added(self, event):
        self.register_task(event["task_added"]["task"])

    async def follow(self):
        async for event in self.subscribe():
            try:
                func = {
                    "SUBSCRIBED": self.handler_subscribed,
                    "TASK_UPDATED": self.handler_task_updated,
                    "FRAMEWORK_ADDED": self.handler_framework_added,
                    "TASK_ADDED": self.handler_task_added,
                }[event["type"]]
            except KeyError:
                log.debug(f"Unknown event type {event['type']}")
                continue

            try:
                func(event)
            except Exception:
                log.error(traceback.format_exc())


def main():
    system_paasta_config = utils.load_system_paasta_config()
    try:
        cluster = system_paasta_config.get_cluster()
    except utils.PaastaNotConfiguredError:
        cluster = None

    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--cluster", type=str, default=cluster, required=(cluster is None)
    )
    args = parser.parse_args()

    loop = asyncio.get_event_loop()
    subscriber = MesosEventSubscriber(cluster=args.cluster)

    loop.run_until_complete(subscriber.follow())


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# Copyright 2019-2020 Yelp Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Usage: ./cleanup_kubernetes_jobs.py [options]

Clean up kubernetes apps that aren't supposed to run on this cluster by deleting them.

Gets the current app list from kubernetes, and then a 'valid_app_list'
via utils.get_services_for_cluster

If an app in the kubernetes app list isn't in the valid_app_list, it's
deleted.

Command line options:

- -d <SOA_DIR>, --soa-dir <SOA_DIR>: Specify a SOA config dir to read from
- -v, --verbose: Verbose output
- -t <KILL_THRESHOLD>, --kill-threshold: The decimal fraction of apps we think
    is sane to kill when this job runs
- -f, --force: Force the killing of apps if we breach the threshold
"""
import argparse
import logging
import sys
import traceback
from contextlib import contextmanager
from typing import Dict
from typing import Generator
from typing import List
from typing import Set
from typing import Tuple

from kubernetes.client import V1Deployment
from kubernetes.client import V1StatefulSet
from pysensu_yelp import Status

from paasta_tools.kubernetes.application.controller_wrappers import DeploymentWrapper
from paasta_tools.kubernetes.application.controller_wrappers import StatefulSetWrapper
from paasta_tools.kubernetes.application.tools import Application
from paasta_tools.kubernetes.application.tools import list_all_applications
from paasta_tools.kubernetes_tools import KubeClient
from paasta_tools.kubernetes_tools import KubernetesDeploymentConfig
from paasta_tools.kubernetes_tools import load_kubernetes_service_config
from paasta_tools.monitoring_tools import send_event
from paasta_tools.utils import _log
from paasta_tools.utils import DEFAULT_SOA_DIR
from paasta_tools.utils import get_services_for_cluster
from paasta_tools.utils import load_system_paasta_config

log = logging.getLogger(__name__)
APPLICATION_TYPES = [V1StatefulSet, V1Deployment]


class DontKillEverythingError(Exception):
    pass


class StatefulSetsAreNotSupportedError(Exception):
    pass


@contextmanager
def alert_state_change(application: Application, cluster: str) -> Generator:
    service = application.kube_deployment.service
    instance = application.kube_deployment.instance
    try:
        yield
        log_line = (
            "Deleted stale Kubernetes apps that looks lost: %s"
            % application.item.metadata.name
        )
        _log(
            service=service,
            component="deploy",
            level="event",
            cluster=cluster,
            instance=instance,
            line=log_line,
        )

    except Exception:
        loglines = ["Exception raised during cleanup of service %s:" % application]
        loglines.extend(traceback.format_exc().rstrip().split("\n"))
        for logline in loglines:
            _log(
                service=service,
                component="deploy",
                level="debug",
                cluster=cluster,
                instance=instance,
                line=logline,
            )
        raise


def instance_is_not_bouncing(
    instance_config: KubernetesDeploymentConfig,
    applications: List[Application],
) -> bool:
    """

    :param instance_config: a KubernetesDeploymentConfig with the configuration of the instance
    :param applications: a list of all deployments or stateful sets on the cluster that match the service
     and instance of provided instance_config
    """
    for application in applications:
        if isinstance(application, DeploymentWrapper):
            existing_app = application.item
            if (
                existing_app.metadata.namespace == instance_config.get_namespace()
                and (
                    instance_config.get_instances()
                    <= (existing_app.status.ready_replicas or 0)
                )
            ) or instance_config.get_desired_state() == "stop":
                return True

        elif (
            isinstance(application, StatefulSetWrapper)
            and application.item.metadata.namespace != instance_config.get_namespace()
        ):
            log.critical(
                "Paasta detected a StatefulSet that was migrated to a new namespace"
                "StatefulSet bouncing across namespaces is not supported"
            )
            raise StatefulSetsAreNotSupportedError
    return False


def get_applications_to_kill(
    applications_dict: Dict[Tuple[str, str], List[Application]],
    cluster: str,
    valid_services: Set[Tuple[str, str]],
    soa_dir: str,
) -> List[Application]:
    """

    :param applications_dict: A dictionary with (service, instance) as keys and a list of applications for each tuple
    :param cluster: paasta cluster
    :param valid_services: a set with the valid (service, instance) tuples for this cluster
    :param soa_dir: The SOA config directory to read from
    :return: list of applications to kill
    """
    log.info("Determining apps to be killed")

    applications_to_kill: List[Application] = []
    for (service, instance), applications in applications_dict.items():
        if len(applications) >= 1:
            if (service, instance) not in valid_services:
                applications_to_kill.extend(applications)
            else:
                instance_config = load_kubernetes_service_config(
                    cluster=cluster, service=service, instance=instance, soa_dir=soa_dir
                )
                try:
                    not_bouncing = instance_is_not_bouncing(
                        instance_config, applications
                    )
                except StatefulSetsAreNotSupportedError:
                    overrides = {
                        "page": True,
                        "alert_after": 0,
                        "tip": f"Revert {service}.{instance} in soa-configs to not include the namespace key.",
                        "runbook": "y/rb-paasta-namespace",
                        "ticket": True,
                    }
                    send_event(
                        service=service,
                        check_name=f"statefulset_bounce_{service}.{instance}",
                        overrides=overrides,
                        status=Status.CRITICAL,  # type: ignore
                        output=f"Unsupported bounce: {service}.{instance}. PaaSTA managed StatefulSets do not support custom namespace",
                        soa_dir=soa_dir,
                    )
                else:
                    for application in applications:
                        if (
                            application.kube_deployment.namespace
                            != instance_config.get_namespace()
                            and not_bouncing
                        ):
                            applications_to_kill.append(application)
    return applications_to_kill


def cleanup_unused_apps(
    soa_dir: str,
    cluster: str,
    kill_threshold: float = 0.5,
    force: bool = False,
) -> None:
    """Clean up old or invalid jobs/apps from kubernetes. Retrieves
    both a list of apps currently in kubernetes and a list of valid
    app ids in order to determine what to kill.

    :param soa_dir: The SOA config directory to read from
    :param cluster: paasta cluster to clean
    :param kill_threshold: The decimal fraction of apps we think is
        sane to kill when this job runs.
    :param force: Force the cleanup if we are above the kill_threshold"""
    log.info("Creating KubeClient")
    kube_client = KubeClient()

    log.info("Loading running Kubernetes apps")
    applications_dict = list_all_applications(kube_client, APPLICATION_TYPES)
    log.info("Retrieving valid apps from yelpsoa_configs")
    valid_services = set(
        get_services_for_cluster(instance_type="kubernetes", soa_dir=soa_dir)
    )

    applications_to_kill: List[Application] = get_applications_to_kill(
        applications_dict, cluster, valid_services, soa_dir
    )

    log.debug("Running apps: %s" % list(applications_dict))
    log.debug("Valid apps: %s" % valid_services)
    log.debug("Terminating: %s" % applications_to_kill)
    if applications_to_kill:
        above_kill_threshold = float(len(applications_to_kill)) / float(
            len(applications_dict)
        ) > float(kill_threshold)
        if above_kill_threshold and not force:
            log.critical(
                "Paasta was about to kill more than %s of the running services, this "
                "is probably a BAD mistake!, run again with --force if you "
                "really need to destroy everything" % kill_threshold
            )
            raise DontKillEverythingError

    for applicaton in applications_to_kill:
        with alert_state_change(applicaton, cluster):
            applicaton.deep_delete(kube_client)


def parse_args(argv):
    parser = argparse.ArgumentParser(description="Cleans up stale kubernetes jobs.")
    parser.add_argument(
        "-d",
        "--soa-dir",
        dest="soa_dir",
        metavar="SOA_DIR",
        default=DEFAULT_SOA_DIR,
        help="define a different soa config directory",
    )
    parser.add_argument(
        "-c",
        "--cluster",
        dest="cluster",
        default=load_system_paasta_config().get_cluster(),
        help="paasta cluster",
    )
    parser.add_argument(
        "-t",
        "--kill-threshold",
        dest="kill_threshold",
        default=0.5,
        help="The decimal fraction of apps we think is "
        "sane to kill when this job runs",
    )
    parser.add_argument(
        "-v", "--verbose", action="store_true", dest="verbose", default=False
    )
    parser.add_argument(
        "-f",
        "--force",
        action="store_true",
        dest="force",
        default=False,
        help="Force the cleanup if we are above the " "kill_threshold",
    )
    return parser.parse_args(argv)


def main(argv=None) -> None:
    args = parse_args(argv)
    soa_dir = args.soa_dir
    kill_threshold = args.kill_threshold
    force = args.force
    cluster = args.cluster
    if args.verbose:
        logging.basicConfig(level=logging.DEBUG)
    else:
        logging.basicConfig(level=logging.WARNING)
    try:
        cleanup_unused_apps(
            soa_dir, cluster=cluster, kill_threshold=kill_threshold, force=force
        )
    except DontKillEverythingError:
        sys.exit(1)


if __name__ == "__main__":
    main()



# coding: utf-8

"""
    Paasta API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from paasta_tools.paastaapi.api_client import ApiClient, Endpoint
from paasta_tools.paastaapi.model_utils import (  # noqa: F401
    check_allowed_values,
    check_validations,
    date,
    datetime,
    file_type,
    none_type,
    validate_and_convert_types
)
from paasta_tools.paastaapi.model.deploy_queue import DeployQueue
from paasta_tools.paastaapi.model.inline_object import InlineObject
from paasta_tools.paastaapi.model.meta_status import MetaStatus


class DefaultApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

        def __delete_service_autoscaler_pause(
            self,
            **kwargs
        ):
            """Unpause the autoscaler  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.delete_service_autoscaler_pause(async_req=True)
            >>> result = thread.get()


            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                None
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            return self.call_with_http_info(**kwargs)

        self.delete_service_autoscaler_pause = Endpoint(
            settings={
                'response_type': None,
                'auth': [],
                'endpoint_path': '/service_autoscaler/pause',
                'operation_id': 'delete_service_autoscaler_pause',
                'http_method': 'DELETE',
                'servers': None,
            },
            params_map={
                'all': [
                ],
                'required': [],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                },
                'attribute_map': {
                },
                'location_map': {
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [],
                'content_type': [],
            },
            api_client=api_client,
            callable=__delete_service_autoscaler_pause
        )

        def __deploy_queue(
            self,
            **kwargs
        ):
            """Get deploy queue contents  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.deploy_queue(async_req=True)
            >>> result = thread.get()


            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                DeployQueue
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            return self.call_with_http_info(**kwargs)

        self.deploy_queue = Endpoint(
            settings={
                'response_type': (DeployQueue,),
                'auth': [],
                'endpoint_path': '/deploy_queue',
                'operation_id': 'deploy_queue',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                ],
                'required': [],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                },
                'attribute_map': {
                },
                'location_map': {
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client,
            callable=__deploy_queue
        )

        def __get_service_autoscaler_pause(
            self,
            **kwargs
        ):
            """Get autoscaling pause time  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.get_service_autoscaler_pause(async_req=True)
            >>> result = thread.get()


            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                str
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            return self.call_with_http_info(**kwargs)

        self.get_service_autoscaler_pause = Endpoint(
            settings={
                'response_type': (str,),
                'auth': [],
                'endpoint_path': '/service_autoscaler/pause',
                'operation_id': 'get_service_autoscaler_pause',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                ],
                'required': [],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                },
                'attribute_map': {
                },
                'location_map': {
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client,
            callable=__get_service_autoscaler_pause
        )

        def __metastatus(
            self,
            cmd_args,
            **kwargs
        ):
            """Get metastatus  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.metastatus(cmd_args, async_req=True)
            >>> result = thread.get()

            Args:
                cmd_args ([str]): comma separated list of command arguments

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                MetaStatus
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['cmd_args'] = \
                cmd_args
            return self.call_with_http_info(**kwargs)

        self.metastatus = Endpoint(
            settings={
                'response_type': (MetaStatus,),
                'auth': [],
                'endpoint_path': '/metastatus',
                'operation_id': 'metastatus',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'cmd_args',
                ],
                'required': [
                    'cmd_args',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'cmd_args':
                        ([str],),
                },
                'attribute_map': {
                    'cmd_args': 'cmd_args',
                },
                'location_map': {
                    'cmd_args': 'query',
                },
                'collection_format_map': {
                    'cmd_args': 'csv',
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client,
            callable=__metastatus
        )

        def __show_version(
            self,
            **kwargs
        ):
            """Version of paasta_tools package  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.show_version(async_req=True)
            >>> result = thread.get()


            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                str
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            return self.call_with_http_info(**kwargs)

        self.show_version = Endpoint(
            settings={
                'response_type': (str,),
                'auth': [],
                'endpoint_path': '/version',
                'operation_id': 'show_version',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                ],
                'required': [],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                },
                'attribute_map': {
                },
                'location_map': {
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client,
            callable=__show_version
        )

        def __update_service_autoscaler_pause(
            self,
            inline_object,
            **kwargs
        ):
            """update_service_autoscaler_pause  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.update_service_autoscaler_pause(inline_object, async_req=True)
            >>> result = thread.get()

            Args:
                inline_object (InlineObject):

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                None
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['inline_object'] = \
                inline_object
            return self.call_with_http_info(**kwargs)

        self.update_service_autoscaler_pause = Endpoint(
            settings={
                'response_type': None,
                'auth': [],
                'endpoint_path': '/service_autoscaler/pause',
                'operation_id': 'update_service_autoscaler_pause',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'inline_object',
                ],
                'required': [
                    'inline_object',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'inline_object':
                        (InlineObject,),
                },
                'attribute_map': {
                },
                'location_map': {
                    'inline_object': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [],
                'content_type': [
                    'application/json'
                ]
            },
            api_client=api_client,
            callable=__update_service_autoscaler_pause
        )



#!/usr/bin/env python
# Copyright 2015-2019 Yelp Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Usage: ./paasta_dump_locally_running_services.py [options]

Outputs a JSON-encoded list of services that are running on this host along
with the host port that each service is listening on.

Command line options:

- -d <SOA_DIR>, --soa-dir <SOA_DIR>: Specify a SOA config dir to read from
"""
import argparse
import json
import sys
from typing import List
from typing import Optional
from typing import Sequence
from typing import Tuple

from paasta_tools.kubernetes_tools import get_kubernetes_services_running_here_for_nerve
from paasta_tools.long_running_service_tools import ServiceNamespaceConfig
from paasta_tools.marathon_tools import get_puppet_services_running_here_for_nerve
from paasta_tools.utils import DEFAULT_SOA_DIR


def parse_args(argv: Optional[Sequence[str]]) -> argparse.Namespace:
    parser = argparse.ArgumentParser(
        description="Dumps information about locally running services."
    )
    parser.add_argument(
        "-d",
        "--soa-dir",
        dest="soa_dir",
        metavar="SOA_DIR",
        default=DEFAULT_SOA_DIR,
        help="define a different soa config directory",
    )
    return parser.parse_args(argv)


def main(argv: Optional[Sequence[str]] = None) -> None:
    args = parse_args(argv)
    soa_dir = args.soa_dir

    service_dump: List[
        Tuple[str, ServiceNamespaceConfig]
    ] = get_puppet_services_running_here_for_nerve(
        soa_dir=soa_dir
    ) + get_kubernetes_services_running_here_for_nerve(
        cluster=None, soa_dir=soa_dir
    )

    print(json.dumps(service_dump))
    sys.exit(0)


if __name__ == "__main__":
    main()
